@@ -89,7 +89,7 @@ mc_dump(ipmi_mc_t *mc, ipmi_cmd_info_t *cmd_info)
 
     ipmi_cmdlang_out_bool(cmd_info, "Active", ipmi_mc_is_active(mc));
     if (ipmi_mc_get_guid(mc, guid) == 0)
-	ipmi_cmdlang_out_binary(cmd_info, "GUID", guid, 16);
+	ipmi_cmdlang_out_binary(cmd_info, "GUID", (char *) guid, 16);
     ipmi_cmdlang_out_int(cmd_info, "SEL Rescan Time",
 			 ipmi_mc_get_sel_rescan_time(mc));
     ipmi_cmdlang_out_bool(cmd_info, "provides_device_sdrs",
@@ -128,7 +128,8 @@ mc_dump(ipmi_mc_t *mc, ipmi_cmd_info_t *cmd_info)
 			 ipmi_mc_manufacturer_id(mc));
     ipmi_cmdlang_out_hex(cmd_info, "product_id", ipmi_mc_product_id(mc));
     ipmi_mc_aux_fw_revision(mc, vals);
-    ipmi_cmdlang_out_binary(cmd_info, "aux_fw_revision", vals, sizeof(vals));
+    ipmi_cmdlang_out_binary(cmd_info, "aux_fw_revision",
+			    (char *) vals, sizeof(vals));
 }
 
 static void
@@ -553,7 +554,8 @@ mc_msg_handler(ipmi_mc_t *mc, ipmi_msg_t *msg, void *cb_data)
     ipmi_cmdlang_out_int(cmd_info, "NetFN", msg->netfn);
     ipmi_cmdlang_out_int(cmd_info, "command", msg->cmd);
     if (msg->data_len)
-	ipmi_cmdlang_out_binary(cmd_info, "Data", msg->data, msg->data_len);
+	ipmi_cmdlang_out_binary(cmd_info, "Data",
+				(char *) msg->data, msg->data_len);
     ipmi_cmdlang_up(cmd_info);
     ipmi_cmdlang_unlock(cmd_info);
 
@@ -663,7 +665,7 @@ sdrs_fetched(ipmi_sdr_info_t *sdrs,
     sdr_info_t      *info = cb_data;
     ipmi_cmd_info_t *cmd_info = info->cmd_info;
     ipmi_cmdlang_t  *cmdlang = ipmi_cmdinfo_get_cmdlang(cmd_info);
-    int             i;
+    unsigned int    i;
     int             rv;
     int             total_size = 0;
 
@@ -697,7 +699,8 @@ sdrs_fetched(ipmi_sdr_info_t *sdrs,
 	snprintf(str, sizeof(str), "%d.%d", sdr.major_version,
 		 sdr.minor_version);
         ipmi_cmdlang_out(cmd_info, "Version", str);
-	ipmi_cmdlang_out_binary(cmd_info, "Data", sdr.data, sdr.length);
+	ipmi_cmdlang_out_binary(cmd_info, "Data",
+				(char *) sdr.data, sdr.length);
 	ipmi_cmdlang_up(cmd_info);
 	total_size += sdr.length+5;
     }
@@ -931,14 +934,11 @@ mc_set_event_log_enable(ipmi_mc_t *mc, void *cb_data)
     
 }
 
-void
-got_chan_info(ipmi_mc_t           *mc,
-	      int                 err,
-	      ipmi_channel_info_t *info,
-	      void                *cb_data)
+static void
+dump_chan_info(ipmi_mc_t           *mc,
+	       ipmi_channel_info_t *info,
+	       ipmi_cmd_info_t     *cmd_info)
 {
-    ipmi_cmd_info_t *cmd_info = cb_data;
-    ipmi_cmdlang_t  *cmdlang = ipmi_cmdinfo_get_cmdlang(cmd_info);
     char            mc_name[IPMI_MC_NAME_LEN];
     char            *str;
     unsigned int    val;
@@ -947,12 +947,6 @@ got_chan_info(ipmi_mc_t           *mc,
 
     ipmi_mc_get_name(mc, mc_name, sizeof(mc_name));
 
-    if (err) {
-	cmdlang->err = err;
-	cmdlang->errstr = "Error getting channel info";
-	goto out_err;
-    }
-
     ipmi_cmdlang_lock(cmd_info);
     ipmi_cmdlang_out(cmd_info, "Channel Info", NULL);
     ipmi_cmdlang_down(cmd_info);
@@ -961,11 +955,17 @@ got_chan_info(ipmi_mc_t           *mc,
     if (!rv)
 	ipmi_cmdlang_out_int(cmd_info, "Channel", val);
     rv = ipmi_channel_info_get_medium(info, &val);
-    if (!rv)
+    if (!rv) {
 	ipmi_cmdlang_out_int(cmd_info, "Medium", val);
+	ipmi_cmdlang_out(cmd_info, "Medium String",
+			 ipmi_channel_medium_string(val));
+    }
     rv = ipmi_channel_info_get_protocol_type(info, &val);
-    if (!rv)
+    if (!rv) {
 	ipmi_cmdlang_out_int(cmd_info, "Protocol Type", val);
+	ipmi_cmdlang_out(cmd_info, "Protocol Type String",
+			 ipmi_channel_protocol_string(val));
+    }
     rv = ipmi_channel_info_get_session_support(info, &val);
     if (!rv) {
 	switch (val) {
@@ -979,12 +979,30 @@ got_chan_info(ipmi_mc_t           *mc,
     }
     rv = ipmi_channel_info_get_vendor_id(info, data);
     if (!rv)
-	ipmi_cmdlang_out_binary(cmd_info, "Vendor ID", data, 3);
+	ipmi_cmdlang_out_binary(cmd_info, "Vendor ID", (char *) data, 3);
     rv = ipmi_channel_info_get_aux_info(info, data);
     if (!rv)
-	ipmi_cmdlang_out_binary(cmd_info, "Aux Info", data, 2);
+	ipmi_cmdlang_out_binary(cmd_info, "Aux Info", (char *) data, 2);
     ipmi_cmdlang_up(cmd_info);
     ipmi_cmdlang_unlock(cmd_info);
+}
+
+static void
+got_chan_info(ipmi_mc_t           *mc,
+	      int                 err,
+	      ipmi_channel_info_t *info,
+	      void                *cb_data)
+{
+    ipmi_cmd_info_t *cmd_info = cb_data;
+    ipmi_cmdlang_t  *cmdlang = ipmi_cmdinfo_get_cmdlang(cmd_info);
+
+    if (err) {
+	cmdlang->err = err;
+	cmdlang->errstr = "Error getting channel info";
+	goto out_err;
+    }
+
+    dump_chan_info(mc, info, cmd_info);
 
  out_err:
     if (cmdlang->err) {
@@ -994,6 +1012,25 @@ got_chan_info(ipmi_mc_t           *mc,
     ipmi_cmdlang_cmd_info_put(cmd_info);
 }
 
+static void
+got_chan_info_multi(ipmi_mc_t           *mc,
+		    int                 err,
+		    ipmi_channel_info_t *info,
+		    void                *cb_data)
+{
+    ipmi_cmd_info_t *cmd_info = cb_data;
+
+    if (err)
+	/* Ignore this on multiple fetches, don't print an error */
+	goto out;
+
+    dump_chan_info(mc, info, cmd_info);
+	    
+
+ out:
+    ipmi_cmdlang_cmd_info_put(cmd_info);
+}
+
 static void
 mc_get_chan_info(ipmi_mc_t *mc, void *cb_data)
 {
@@ -1007,26 +1044,38 @@ mc_get_chan_info(ipmi_mc_t *mc, void *cb_data)
 
 
     if ((argc - curr_arg) < 1) {
-	/* Not enough parameters */
-	cmdlang->errstr = "Not enough parameters";
-	cmdlang->err = EINVAL;
-	goto out_err;
-    }
+	int count = 0;
+	/* List them all */
+	for (channel=0; channel<8; channel++) {
+	    ipmi_cmdlang_cmd_info_get(cmd_info);
+	    rv = ipmi_mc_channel_get_info(mc, channel, got_chan_info_multi,
+					  cmd_info);
+	    if (rv)
+		ipmi_cmdlang_cmd_info_put(cmd_info);
+	    else
+		count++;
+	}
+	if (count == 0) {
+	    cmdlang->err = rv;
+	    cmdlang->errstr = "Could not get channel info for any channels";
+	    goto out_err;
+	}
+    } else {
+	ipmi_cmdlang_get_int(argv[curr_arg], &channel, cmd_info);
+	if (cmdlang->err) {
+	    cmdlang->errstr = "channel invalid";
+	    goto out_err;
+	}
+	curr_arg++;
 
-    ipmi_cmdlang_get_int(argv[curr_arg], &channel, cmd_info);
-    if (cmdlang->err) {
-	cmdlang->errstr = "channel invalid";
-	goto out_err;
-    }
-    curr_arg++;
-
-    ipmi_cmdlang_cmd_info_get(cmd_info);
-    rv = ipmi_mc_channel_get_info(mc, channel, got_chan_info, cmd_info);
-    if (rv) {
-	ipmi_cmdlang_cmd_info_put(cmd_info);
-	cmdlang->err = rv;
-	cmdlang->errstr = "Could not get channel info";
-	goto out_err;
+	ipmi_cmdlang_cmd_info_get(cmd_info);
+	rv = ipmi_mc_channel_get_info(mc, channel, got_chan_info, cmd_info);
+	if (rv) {
+	    ipmi_cmdlang_cmd_info_put(cmd_info);
+	    cmdlang->err = rv;
+	    cmdlang->errstr = "Could not get channel info";
+	    goto out_err;
+	}
     }
 
     return;
@@ -1543,7 +1592,7 @@ got_users(ipmi_mc_t        *mc,
     ipmi_cmdlang_t  *cmdlang = ipmi_cmdinfo_get_cmdlang(cmd_info);
     ipmi_user_t     *user;
     char            mc_name[IPMI_MC_NAME_LEN];
-    int             i, j, k;
+    int		    i, j, k;
     char            str[17];
     unsigned int    count;
     unsigned int    channel;
@@ -1579,7 +1628,7 @@ got_users(ipmi_mc_t        *mc,
     rv = ipmi_user_list_get_fixed_users(list, &val);
     if (!rv)
 	ipmi_cmdlang_out_int(cmd_info, "Fixed Users", val);
-    for (i=0; i<count; i++) {
+    for (i=0; i<(int)count; i++) {
 	user = ipmi_user_list_get_user(list, i);
 	if (!user)
 	    continue;
@@ -2020,7 +2069,39 @@ mc_active(ipmi_mc_t *mc, int active, void *cb_data)
 
  out_err:
     ipmi_cmdlang_global_err(mc_name,
-			    "cmd_mc.c(presence_change)",
+			    "cmd_mc.c(mc_active)",
+			    errstr, rv);
+    if (evi)
+	ipmi_cmdlang_cmd_info_put(evi);
+}
+
+static void
+mc_fully_up(ipmi_mc_t *mc, void *cb_data)
+{
+    char            *errstr;
+    int             rv;
+    ipmi_cmd_info_t *evi;
+    char            mc_name[IPMI_MC_NAME_LEN];
+
+    ipmi_mc_get_name(mc, mc_name, sizeof(mc_name));
+
+    evi = ipmi_cmdlang_alloc_event_info();
+    if (!evi) {
+	rv = ENOMEM;
+	errstr = "Out of memory";
+	goto out_err;
+    }
+
+    ipmi_cmdlang_out(evi, "Object Type", "MC");
+    ipmi_cmdlang_out(evi, "Name", mc_name);
+    ipmi_cmdlang_out(evi, "Operation", "Fully Up");
+
+    ipmi_cmdlang_cmd_info_put(evi);
+    return;
+
+ out_err:
+    ipmi_cmdlang_global_err(mc_name,
+			    "cmd_mc.c(mc_fully_up)",
 			    errstr, rv);
     if (evi)
 	ipmi_cmdlang_cmd_info_put(evi);
@@ -2059,6 +2140,11 @@ ipmi_cmdlang_mc_change(enum ipmi_update_e op,
 	    errstr = "ipmi_mc_add_active_handler failed";
 	    goto out_err;
 	}
+	rv = ipmi_mc_add_fully_up_handler(mc, mc_fully_up, NULL);
+	if (rv) {
+	    errstr = "ipmi_mc_add_fully_up_handler failed";
+	    goto out_err;
+	}
 	break;
 
 	case IPMI_DELETED:
