@@ -31,8 +31,8 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifndef _IPMI_SDR_H
-#define _IPMI_SDR_H
+#ifndef OPENIPMI_SDR_H
+#define OPENIPMI_SDR_H
 #include <OpenIPMI/ipmi_types.h>
 #include <stdint.h>
 
@@ -70,6 +70,10 @@ int ipmi_sdr_info_alloc(ipmi_domain_t   *domain,
 /* Remove all the SDRs, but don't destroy the SDR repository. */
 void ipmi_sdr_clean_out_sdrs(ipmi_sdr_info_t *sdrs);
 
+/* Stop any timer operation; if the MC is in shutdown this should halt
+   any running operations. */
+void ipmi_sdr_cleanout_timer(ipmi_sdr_info_t *sdrs);
+
 /* Destroy an SDR.  Note that if the SDR is currently fetching SDRs,
    the destroy cannot complete immediatly, it will be marked for
    destruction later.  You can supply a callback that, if not NULL,
@@ -156,4 +160,4 @@ int ipmi_sdr_save(ipmi_sdr_info_t  *sdrs,
 }
 #endif
 
-#endif /* _IPMI_SDR_H */
+#endif /* OPENIPMI_SDR_H */
