@@ -31,8 +31,8 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifndef _IPMI_UI_H
-#define _IPMI_UI_H
+#ifndef OPENIPMI_UI_H
+#define OPENIPMI_UI_H
 
 #include <OpenIPMI/ipmi_types.h>
 #include <OpenIPMI/os_handler.h>
@@ -44,7 +44,7 @@ extern "C" {
 
 extern os_handler_t ipmi_ui_cb_handlers;
 
-int ipmi_ui_init(selector_t **selector, int full_screen);
+int ipmi_ui_init(os_handler_t *os_hnd, int full_screen);
 void ipmi_ui_shutdown(void);
 
 void ipmi_ui_set_first_domain(ipmi_domain_id_t fdomain_id);
@@ -60,4 +60,4 @@ void ipmi_ui_setup_done(ipmi_domain_t *mc,
 }
 #endif
 
-#endif /* _IPMI_UI_H */
+#endif /* OPENIPMI_UI_H */
