@@ -28,7 +28,24 @@
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this program; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+ *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * Modified BSD Licence
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *   1. Redistributions of source code must retain the above copyright
+ *      notice, this list of conditions and the following disclaimer.
+ *   2. Redistributions in binary form must reproduce the above
+ *      copyright notice, this list of conditions and the following
+ *      disclaimer in the documentation and/or other materials provided
+ *      with the distribution.
+ *   3. The name of the author may not be used to endorse or promote
+ *      products derived from this software without specific prior
+ *      written permission.
+ */
 
 #include "bmc.h"
 
@@ -39,6 +56,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <time.h>
 
 #include <OpenIPMI/ipmi_err.h>
 #include <OpenIPMI/ipmi_msgbits.h>
@@ -1616,8 +1634,15 @@ fru_sem_trywait(fru_data_t *fru)
 
     /* Wait 250ms for the semaphore. */
   restart:
-    ts.tv_sec = 0;
-    ts.tv_nsec = 250000000;
+    rv = clock_gettime(CLOCK_REALTIME, &ts);
+    if (rv == -1)
+	return errno;
+
+    ts.tv_nsec += 250000000;
+    if (ts.tv_nsec >= 1000000000) {
+	ts.tv_nsec -= 1000000000;
+	ts.tv_sec += 1;
+    }
     rv = sem_timedwait(&fru->sem, &ts);
     if (rv) {
 	if (rv == EINTR)
