@@ -109,8 +109,6 @@ read_persist_users(sys_data_t *sys)
 		users[j].privilege = iv;
 	    if (!read_persist_int(p, &iv, "%d.max_sessions", j))
 		users[j].max_sessions = iv;
-	    if (!read_persist_int(p, &iv, "%d.allowed_auths", j))
-		users[j].allowed_auths = iv;
 	}
 	free_persist(p);
     }
@@ -144,7 +142,6 @@ write_persist_users(sys_data_t *sys)
 			     "%d.passwd", j);
 	    add_persist_int(p, users[j].privilege, "%d.privilege", j);
 	    add_persist_int(p, users[j].max_sessions, "%d.max_sessions", j);
-	    add_persist_int(p, users[j].allowed_auths, "%d.allowed_auths", j);
 	}
 	write_persist(p);
 	free_persist(p);
@@ -158,8 +155,11 @@ struct variable {
     struct variable *next;
 } *vars;
 
+/*
+ *
+ */
 int
-add_variable(const char *name, const char *value)
+add_variable(const char *name, char *value)
 {
     struct variable *var = vars, *last = NULL;
 
@@ -176,8 +176,10 @@ add_variable(const char *name, const char *value)
 	if (!var)
 	    return ENOMEM;
 	var->name = strdup(name);
-	if (!var->name)
+	if (!var->name) {
+	    free(var);
 	    return ENOMEM;
+	}
 	var->next = NULL;
 	if (last)
 	    last->next = var;
@@ -185,7 +187,7 @@ add_variable(const char *name, const char *value)
 	    vars = var;
     }
     
-    var->value = strdup(value);
+    var->value = value;
     if (!var->value)
 	return ENOMEM;
 
@@ -272,7 +274,7 @@ isquote(char c)
 }
 
 int
-get_delim_str(char **rtokptr, const char **rval, const char **err)
+get_delim_str(char **rtokptr, char **rval, const char **err)
 {
     char *tokptr = *rtokptr;
     char endc;
@@ -299,8 +301,12 @@ get_delim_str(char **rtokptr, const char **rval, const char **err)
 	    oldc = *tokptr;
 	    *tokptr = '\0';
 	    val = find_variable(val);
-	    if (!val)
+	    if (!val) {
+		if (rv)
+		    free(rv);
+		*err = "unable to find variable";
 		return -1;
+	    }
 	    *tokptr = oldc;
 	} else if (isquote(*tokptr)) {
 	    endc = *tokptr;
@@ -316,6 +322,8 @@ get_delim_str(char **rtokptr, const char **rval, const char **err)
 	    *tokptr = '\0';
 	    tokptr++;
 	} else {
+	    if (rv)
+		free(rv);
 	    *err = "string value must start with '\"' or '''";
 	    return -1;
 	}
@@ -323,6 +331,7 @@ get_delim_str(char **rtokptr, const char **rval, const char **err)
 	if (rv) {
 	    char *newrv = malloc(strlen(rv) + strlen(val) + 1);
 	    if (!newrv) {
+		free(rv);
 		*err = "Out of memory copying string";
 		return -1;
 	    }
@@ -648,11 +657,6 @@ get_user(char **tokptr, sys_data_t *sys, const char **err)
 	return rv;
     sys->cusers[num].max_sessions = val;
 
-    rv = get_auths(tokptr, &val, err);
-    if (rv)
-	return rv;
-    sys->cusers[num].allowed_auths = val;
-
     return 0;
 }
 
@@ -686,6 +690,7 @@ load_dynamic_libs(sys_data_t *sys, int print_version)
 	    if (func) {
 		err = func(sys, dlib->init);
 		if (err) {
+		    dlclose(handle);
 		    fprintf(stderr, "Error from module %s version print: %s\n",
 			    dlib->file, strerror(err));
 		    return EINVAL;
@@ -697,6 +702,7 @@ load_dynamic_libs(sys_data_t *sys, int print_version)
 	    if (func) {
 		err = func(sys, dlib->init);
 		if (err) {
+		    dlclose(handle);
 		    fprintf(stderr, "Error from module %s init: %s\n",
 			    dlib->file, strerror(err));
 		    return EINVAL;
@@ -753,7 +759,8 @@ read_config(sys_data_t *sys,
 	    continue;
 
 	if (strcmp(tok, "define") == 0) {
-	    const char *varname, *value;
+	    const char *varname;
+	    char *value;
 
 	    varname = mystrtok(NULL, " \t\n", &tokptr);
 	    if (!varname) {
@@ -766,12 +773,13 @@ read_config(sys_data_t *sys,
 		goto next;
 	    err = add_variable(varname, value);
 	    if (err) {
+		free(value);
 		err = ENOMEM;
 		errstr = "Out of memory";
 		goto next;
 	    }
 	} else if (strcmp(tok, "loadlib") == 0) {
-	    const char *library = NULL, *initstr = NULL;
+	    char *library = NULL, *initstr = NULL;
 	    struct dliblist *dlib, *dlibp;
 
 	    err = get_delim_str(&tokptr, &library, &errstr);
@@ -824,7 +832,7 @@ read_config(sys_data_t *sys,
 	} else if (strcmp(tok, "sol") == 0) {
 	    err = sol_read_config(&tokptr, sys, &errstr);
 	} else if (strcmp(tok, "chassis_control") == 0) {
-	    const char *prog;
+	    char *prog;
 	    err = get_delim_str(&tokptr, &prog, &errstr);
 	    if (!err)
 		ipmi_set_chassis_control_prog(sys->mc, prog);
