@@ -1,3 +1,57 @@
+/*
+ * config.c
+ *
+ * MontaVista IPMI code for reading lanserv configuration files.
+ *
+ * Author: MontaVista Software, Inc.
+ *         Corey Minyard <minyard@mvista.com>
+ *         source@mvista.com
+ *
+ * Copyright 2003,2004,2005,2012,2016 MontaVista Software Inc.
+ *
+ * This software is available to you under a choice of one of two
+ * licenses.  You may choose to be licensed under the terms of the GNU
+ * Lesser General Public License (GPL) Version 2 or the modified BSD
+ * license below.  The following disclamer applies to both licenses:
+ *
+ *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
+ *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+ *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ *  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
+ *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
+ *  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
+ *  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * GNU Lesser General Public Licence
+ *
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this program; if not, write to the Free
+ *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * Modified BSD Licence
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *   1. Redistributions of source code must retain the above copyright
+ *      notice, this list of conditions and the following disclaimer.
+ *   2. Redistributions in binary form must reproduce the above
+ *      copyright notice, this list of conditions and the following
+ *      disclaimer in the documentation and/or other materials provided
+ *      with the distribution.
+ *   3. The name of the author may not be used to endorse or promote
+ *      products derived from this software without specific prior
+ *      written permission.
+ */
 
 #include <string.h>
 #include <stdio.h>
@@ -22,7 +76,7 @@ emu_get_uchar(emu_out_t *out, char **toks, unsigned char *val, char *errstr,
 	if (empty_ok)
 	    return ENOSPC;
 	if (errstr)
-	    out->printf(out, "**No %s given\n", errstr);
+	    out->eprintf(out, "**No %s given\n", errstr);
 	return EINVAL;
     }
     if (str[0] == '\'') {
@@ -32,7 +86,7 @@ emu_get_uchar(emu_out_t *out, char **toks, unsigned char *val, char *errstr,
     *val = strtoul(str, &tmpstr, 0);
     if (*tmpstr != '\0') {
 	if (errstr)
-	    out->printf(out, "**Invalid %s given\n", errstr);
+	    out->eprintf(out, "**Invalid %s given\n", errstr);
 	return EINVAL;
     }
 
@@ -54,7 +108,7 @@ emu_get_uchar_with_vals(emu_out_t *out, char **toks,
 	if (empty_ok)
 	    return ENOSPC;
 	if (errstr)
-	    out->printf(out, "**No %s given\n", errstr);
+	    out->eprintf(out, "**No %s given\n", errstr);
 	return EINVAL;
     }
     if (str[0] == '\'') {
@@ -77,7 +131,7 @@ emu_get_uchar_with_vals(emu_out_t *out, char **toks,
     *val = strtoul(str, &tmpstr, 0);
     if (*tmpstr != '\0') {
 	if (errstr)
-	    out->printf(out, "**Invalid %s given\n", errstr);
+	    out->eprintf(out, "**Invalid %s given\n", errstr);
 	return EINVAL;
     }
  out:
@@ -96,12 +150,12 @@ emu_get_bitmask(emu_out_t *out, char **toks, uint16_t *val, char *errstr,
 	if (empty_ok)
 	    return ENOSPC;
 	if (errstr)
-	    out->printf(out, "**No %s given\n", errstr);
+	    out->eprintf(out, "**No %s given\n", errstr);
 	return EINVAL;
     }
     if (strlen(str) != size) {
 	if (errstr)
-	    out->printf(out, "**invalid number of bits in %s\n", errstr);
+	    out->eprintf(out, "**invalid number of bits in %s\n", errstr);
 	return EINVAL;
     }
     *val = 0;
@@ -112,7 +166,7 @@ emu_get_bitmask(emu_out_t *out, char **toks, uint16_t *val, char *errstr,
 	    *val |= 1 << i;
 	} else {
 	    if (errstr)
-		out->printf(out, "**Invalid bit value '%c' in %s\n", str[j],
+		out->eprintf(out, "**Invalid bit value '%c' in %s\n", str[j],
 			    errstr);
 	    return EINVAL;
 	}
@@ -130,13 +184,13 @@ emu_get_uint(emu_out_t *out, char **toks, unsigned int *val, char *errstr)
     str = mystrtok(NULL, " \t\n", toks);
     if (!str) {
 	if (errstr)
-	    out->printf(out, "**No %s given\n", errstr);
+	    out->eprintf(out, "**No %s given\n", errstr);
 	return EINVAL;
     }
     *val = strtoul(str, &tmpstr, 0);
     if (*tmpstr != '\0') {
 	if (errstr)
-	    out->printf(out, "**Invalid %s given\n", errstr);
+	    out->eprintf(out, "**Invalid %s given\n", errstr);
 	return EINVAL;
     }
 
@@ -152,7 +206,7 @@ emu_get_bytes(emu_out_t *out, char **tokptr, unsigned char *data, char *errstr,
 
     if (!tok) {
 	if (errstr)
-	    out->printf(out, "**No %s given\n", errstr);
+	    out->eprintf(out, "**No %s given\n", errstr);
 	return EINVAL;
     }
     if (*tok == '"') {
@@ -161,7 +215,7 @@ emu_get_bytes(emu_out_t *out, char **tokptr, unsigned char *data, char *errstr,
 	tok++;
 	end = strlen(tok) - 1;
 	if (tok[end] != '"') {
-	  out->printf(out, "**ASCII %s doesn't end in '\"'", errstr);
+	  out->eprintf(out, "**ASCII %s doesn't end in '\"'", errstr);
 	    return EINVAL;
 	}
 	if (end > (len - 1))
@@ -174,7 +228,7 @@ emu_get_bytes(emu_out_t *out, char **tokptr, unsigned char *data, char *errstr,
 	char         c[3];
 	/* HEX pw */
 	if (strlen(tok) != 32) {
-	    out->printf(out, "**HEX %s not 32 HEX characters long", errstr);
+	    out->eprintf(out, "**HEX %s not 32 HEX characters long", errstr);
 	    return EINVAL;
 	}
 	c[2] = '\0';
@@ -185,7 +239,7 @@ emu_get_bytes(emu_out_t *out, char **tokptr, unsigned char *data, char *errstr,
 	    tok++;
 	    data[i] = strtoul(c, &end, 16);
 	    if (*end != '\0') {
-		out->printf(out, "**Invalid HEX character in %s", errstr);
+		out->eprintf(out, "**Invalid HEX character in %s", errstr);
 		return -1;
 	    }
 	}
@@ -209,12 +263,12 @@ read_command_file(emu_out_t *out, emu_data_t *emu, const char *command_file)
 
 	buffer = malloc(INPUT_BUFFER_SIZE);
 	if (!buffer) {
-	    out->printf(out, "Could not allocate buffer memory\n");
+	    out->eprintf(out, "Could not allocate buffer memory\n");
 	    rv = ENOMEM;
 	    goto out;
 	}
 	while (fgets(buffer+pos, INPUT_BUFFER_SIZE-pos, f)) {
-	    out->printf(out, "%s", buffer+pos);
+	    out->eprintf(out, "%s", buffer+pos);
 	    if (buffer[pos] == '#')
 		continue;
 	    pos = strlen(buffer);
@@ -264,7 +318,7 @@ sel_enable(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
     rv = ipmi_mc_enable_sel(mc, max_records, flags);
     if (rv)
-	out->printf(out, "**Unable to enable sel, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to enable sel, error 0x%x\n", rv);
     return rv;
 }
 
@@ -289,9 +343,9 @@ sel_add(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
     rv = ipmi_mc_add_to_sel(mc, record_type, data, &r);
     if (rv)
-	out->printf(out, "**Unable to add to sel, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to add to sel, error 0x%x\n", rv);
     else
-	out->printf(out, "Added record %d\n", r);
+	out->eprintf(out, "Added record %d\n", r);
     return rv;
 }
 
@@ -307,14 +361,14 @@ main_sdr_add(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	if (rv == ENOSPC)
 	    break;
 	if (rv) {
-	    out->printf(out, "**Error 0x%x in data byte %d\n", rv, i);
+	    out->eprintf(out, "**Error 0x%x in data byte %d\n", rv, i);
 	    return rv;
 	}
     }
 
     rv = ipmi_mc_add_main_sdr(mc, data, i);
     if (rv)
-	out->printf(out, "**Unable to add to sdr, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to add to sdr, error 0x%x\n", rv);
     return rv;
 }
 
@@ -335,14 +389,14 @@ device_sdr_add(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	if (rv == ENOSPC)
 	    break;
 	if (rv) {
-	    out->printf(out, "**Error 0x%x in data byte %d\n", rv, i);
+	    out->eprintf(out, "**Error 0x%x in data byte %d\n", rv, i);
 	    return rv;
 	}
     }
 
     rv = ipmi_mc_add_device_sdr(mc, lun, data, i);
     if (rv)
-	out->printf(out, "**Unable to add to sdr, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to add to sdr, error 0x%x\n", rv);
     return rv;
 }
 
@@ -385,7 +439,7 @@ sensor_add(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
     while (tok) {
 	if (strcmp(tok, "poll") == 0) {
 	    if (handler) {
-		out->printf(out, "**poll given twice in sensor\n", tok);
+		out->eprintf(out, "**poll given twice in sensor\n", tok);
 		return -1;
 	    }
 
@@ -395,27 +449,27 @@ sensor_add(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
 	    tok = mystrtok(NULL, " \t\n", toks);
 	    if (!tok) {
-		out->printf(out, "**No polled sensor handler given\n", tok);
+		out->eprintf(out, "**No polled sensor handler given\n", tok);
 		return -1;
 	    }
 
 	    handler = ipmi_sensor_find_handler(tok);
 	    if (!handler) {
-		out->printf(out, "**Invalid sensor handler: %s\n", tok);
+		out->eprintf(out, "**Invalid sensor handler: %s\n", tok);
 		return -1;
 	    }
 
 	    rv = handler->init(mc, lun, num, toks, handler->cb_data, &rcb_data,
 			       &errstr);
 	    if (rv) {
-		out->printf(out, "**Error initializing sensor handler: %s\n", 
+		out->eprintf(out, "**Error initializing sensor handler: %s\n",
 			    errstr);
 		return rv;
 	    }
 	} else if (strcmp(tok, "event-only") == 0) {
 	    event_only = 1;
 	} else {
-	    out->printf(out, "**Unknown sensor option: %s\n", tok);
+	    out->eprintf(out, "**Unknown sensor option: %s\n", tok);
 	    return -1;
 	}
 
@@ -423,7 +477,7 @@ sensor_add(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
     }
 
     if (handler && event_only) {
-	out->printf(out, "**An event-only sensor cannot be polled\n");
+	out->eprintf(out, "**An event-only sensor cannot be polled\n");
 	return -1;
     }
 
@@ -434,7 +488,7 @@ sensor_add(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	if (!rv && handler->postinit) {
 	    rv = handler->postinit(rcb_data, &errstr);
 	    if (rv) {
-		out->printf(out, "**Error in sensor handler postinit: %s\n", 
+		out->eprintf(out, "**Error in sensor handler postinit: %s\n",
 			    errstr);
 	    }
 	}
@@ -442,7 +496,7 @@ sensor_add(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	rv = ipmi_mc_add_sensor(mc, lun, num, type, code, event_only);
     }
     if (rv)
-	out->printf(out, "**Unable to add to sensor, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to add to sensor, error 0x%x\n", rv);
     return rv;
 }
 
@@ -478,7 +532,7 @@ sensor_set_bit(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
     rv = ipmi_mc_sensor_set_bit(mc, lun, num, bit, value, gen_event);
     if (rv)
-	out->printf(out, "**Unable to set sensor bit, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to set sensor bit, error 0x%x\n", rv);
     return rv;
 }
 
@@ -509,7 +563,7 @@ sensor_set_bit_clr_rest(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **
 
     rv = ipmi_mc_sensor_set_bit_clr_rest(mc, lun, num, bit, gen_event);
     if (rv)
-	out->printf(out, "**Unable to set sensor bit, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to set sensor bit, error 0x%x\n", rv);
     return rv;
 }
 
@@ -540,7 +594,7 @@ sensor_set_value(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
     rv = ipmi_mc_sensor_set_value(mc, lun, num, value, gen_event);
     if (rv)
-	out->printf(out, "**Unable to set sensor value, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to set sensor value, error 0x%x\n", rv);
     return rv;
 }
 
@@ -583,7 +637,7 @@ sensor_set_hysteresis(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **to
     rv = ipmi_mc_sensor_set_hysteresis(mc, lun, num, support, positive,
 				       negative);
     if (rv)
-	out->printf(out, "**Unable to set sensor hysteresis, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to set sensor hysteresis, error 0x%x\n", rv);
     return rv;
 }
 
@@ -628,7 +682,7 @@ sensor_set_threshold(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **tok
     rv = ipmi_mc_sensor_set_threshold(mc, lun, num, support,
 				      enabled, 1, thresholds);
     if (rv)
-	out->printf(out, "**Unable to set sensor thresholds, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to set sensor thresholds, error 0x%x\n", rv);
     return rv;
 }
 
@@ -705,7 +759,7 @@ sensor_set_event_support(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char *
 					  assert_support, deassert_support,
 					  assert_enabled, deassert_enabled);
     if (rv)
-	out->printf(out, "**Unable to set sensor thresholds, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to set sensor thresholds, error 0x%x\n", rv);
     return rv;
 }
 
@@ -765,7 +819,7 @@ mc_add(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	else if (strcmp("persist_sdr", tok) == 0)
 	    flags |= IPMI_MC_PERSIST_SDR;
 	else {
-	    out->printf(out, "**Invalid MC flag: %s\n", tok);
+	    out->eprintf(out, "**Invalid MC flag: %s\n", tok);
 	    return -1;
 	}
     }
@@ -779,7 +833,7 @@ mc_add(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 			 device_revision, major_fw_rev, minor_fw_rev,
 			 device_support, mfg_id, product_id, flags);
     if (rv)
-	out->printf(out, "**Unable to add the MC, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to add the MC, error 0x%x\n", rv);
     return rv;
 }
 
@@ -836,7 +890,7 @@ mc_set_power(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
     rv = ipmi_mc_set_power(mc, power, gen_int);
     if (rv)
-	out->printf(out, "**Unable to set power, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to set power, error 0x%x\n", rv);
     return rv;
 }
 
@@ -862,24 +916,26 @@ mc_add_fru_data(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
     tok = mystrtok(NULL, " \t\n", toks);
     if (!tok) {
-	out->printf(out, "**No FRU data type given");
+	out->eprintf(out, "**No FRU data type given");
 	return -1;
     }
     if (strcmp(tok, "file") == 0) {
 	unsigned int file_offset;
+	char *frufn;
 
 	rv = emu_get_uint(out, toks, &file_offset, "file offset");
 	if (rv)
 	    return rv;
 
-	rv = get_delim_str(toks, &tok, &errstr);
+	rv = get_delim_str(toks, &frufn, &errstr);
 	if (rv) {
-	    out->printf(out, "**Error with FRU filename: %d", strerror(rv));
+	    out->eprintf(out, "**Error with FRU filename: %d", strerror(rv));
 	    return rv;
 	}
-	rv = ipmi_mc_add_fru_file(mc, devid, length, file_offset, (void *) tok);
+	rv = ipmi_mc_add_fru_file(mc, devid, length, file_offset,
+				  (void *) frufn);
 	if (rv)
-	    out->printf(out, "**Unable to add FRU file, error 0x%x\n", rv);
+	    out->eprintf(out, "**Unable to add FRU file, error 0x%x\n", rv);
 	
     } else if (strcmp(tok, "data") == 0) {
 	for (i=0; i<length; i++) {
@@ -887,14 +943,14 @@ mc_add_fru_data(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	    if (rv == ENOSPC)
 		break;
 	    if (rv) {
-		out->printf(out, "**Error 0x%x in data byte %d\n", rv, i);
+		out->eprintf(out, "**Error 0x%x in data byte %d\n", rv, i);
 		return rv;
 	    }
 	}
 
 	rv = emu_get_uchar(out, toks, &data[i], "data byte", 1);
 	if (rv != ENOSPC) {
-	    out->printf(out, "**Error: input data too long for FRU\n", rv, i);
+	    out->eprintf(out, "**Error: input data too long for FRU\n", rv, i);
 	    return EINVAL;
 	}
 
@@ -902,9 +958,9 @@ mc_add_fru_data(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
 	rv = ipmi_mc_add_fru_data(mc, devid, length, NULL, data);
 	if (rv)
-	    out->printf(out, "**Unable to add FRU data, error 0x%x\n", rv);
+	    out->eprintf(out, "**Unable to add FRU data, error 0x%x\n", rv);
     } else {
-	out->printf(out, "**FRU type not given, need file or data\n");
+	out->eprintf(out, "**FRU type not given, need file or data\n");
 	rv = EINVAL;
     }
     return rv;
@@ -925,28 +981,28 @@ mc_dump_fru_data(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
     rv = ipmi_mc_get_fru_data_len(mc, devid, &length);
     if (rv) {
-	out->printf(out, "**Unable to dump FRU data, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to dump FRU data, error 0x%x\n", rv);
 	goto out;
     }
 
     data = malloc(length);
     if (!data) {
-	out->printf(out, "**Unable to dump FRU data, out of memory\n", rv);
+	out->eprintf(out, "**Unable to dump FRU data, out of memory\n", rv);
 	goto out;
     }
 
     rv = ipmi_mc_get_fru_data(mc, devid, length, data);
     if (rv) {
-	out->printf(out, "**Unable to dump FRU data, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to dump FRU data, error 0x%x\n", rv);
 	goto out;
     }
 
     for (i=0; i<length; i++) {
 	if ((i > 0) && ((i % 8) == 0))
-	    out->printf(out, "\n");
-	out->printf(out, " 0x%2.2x", data[i]);
+	    out->eprintf(out, "\n");
+	out->eprintf(out, " 0x%2.2x", data[i]);
     }
-    out->printf(out, "\n");
+    out->eprintf(out, "\n");
 
  out:
     if (data)
@@ -965,7 +1021,7 @@ mc_setbmc(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	return rv;
     rv = ipmi_emu_set_bmc_mc(emu, ipmb);
     if (rv)
-	out->printf(out, "**Invalid IPMB address\n");
+	out->eprintf(out, "**Invalid IPMB address\n");
     return rv;
 }
 
@@ -976,7 +1032,7 @@ atca_enable(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
     rv = ipmi_emu_atca_enable(emu);
     if (rv)
-	out->printf(out, "**Unable to enable ATCA mode, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to enable ATCA mode, error 0x%x\n", rv);
     return rv;
 }
 
@@ -1000,7 +1056,7 @@ atca_set_site(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
     rv = ipmi_emu_atca_set_site(emu, hw_address, site_type, site_number);
     if (rv)
-	out->printf(out, "**Unable to set site type, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to set site type, error 0x%x\n", rv);
     return rv;
 }
 
@@ -1016,19 +1072,20 @@ mc_set_num_leds(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 
     rv = ipmi_mc_set_num_leds(mc, count);
     if (rv)
-	out->printf(out, "**Unable to set number of LEDs, error 0x%x\n", rv);
+	out->eprintf(out, "**Unable to set number of LEDs, error 0x%x\n", rv);
     return rv;
 }
 
 static int
 read_cmds(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 {
-    const char *filename, *errstr;
+    char *filename;
+    const char *errstr;
     int err;
 
     err = get_delim_str(toks, &filename, &errstr);
     if (err) {
-	out->printf(out, "Could not get include filename: %s\n", errstr);
+	out->eprintf(out, "Could not get include filename: %s\n", errstr);
 	return err;
     }
 
@@ -1040,7 +1097,7 @@ read_cmds(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
     {
 	char *nf = malloc(strlen(BASE_CONF_STR) + strlen(filename) + 2);
 	if (!nf) {
-	    out->printf(out, "Out of memory in include\n", errstr);
+	    out->eprintf(out, "Out of memory in include\n", errstr);
 	    goto out_err;
 	}
 	strcpy(nf, BASE_CONF_STR);
@@ -1050,7 +1107,7 @@ read_cmds(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	filename = nf;
 	err = read_command_file(out, emu, filename);
 	if (err) {
-	    out->printf(out, "Could not read include file %s\n", filename);
+	    out->eprintf(out, "Could not read include file %s\n", filename);
 	}
     }
 
@@ -1087,8 +1144,10 @@ debug_cmd(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	    level |= DEBUG_RAW_MSG;
 	} else if (strcmp(tok, "msg") == 0) {
 	    level |= DEBUG_MSG;
+	} else if (strcmp(tok, "sol") == 0) {
+	    level |= DEBUG_SOL;
 	} else {
-	    out->printf(out, "Invalid debug level '%s', options are 'raw' and 'msg'\n",
+	    out->eprintf(out, "Invalid debug level '%s', options are 'raw' and 'msg'\n",
 		   tok);
 	    return EINVAL;
 	}
@@ -1109,7 +1168,7 @@ persist_cmd(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 	} else if (strcmp(tok, "off") == 0) {
 	    persist_enable = 0;
 	} else {
-	    out->printf(out, "Invalid persist vale '%s', options are 'on' and 'off'\n",
+	    out->eprintf(out, "Invalid persist vale '%s', options are 'on' and 'off'\n",
 		   tok);
 	    return EINVAL;
 	}
@@ -1128,23 +1187,25 @@ quit(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 static int
 do_define(emu_out_t *out, emu_data_t *emu, lmc_data_t *mc, char **toks)
 {
-    const char *name, *value;
+    const char *name;
+    char *value;
     int err;
     const char *errstr;
 
     name = mystrtok(NULL, " \t\n", toks);
     if (!name) {
-	out->printf(out, "No variable name given for define\n");
+	out->eprintf(out, "No variable name given for define\n");
 	return EINVAL;
     }
     err = get_delim_str(toks, &value, &errstr);
     if (err) {
-	out->printf(out, "Could not get variable %s value: %s\n", name, errstr);
+	out->eprintf(out, "Could not get variable %s value: %s\n", name, errstr);
 	return err;
     }
     err = add_variable(name, value);
     if (err) {
-	out->printf(out, "Out of memory setting variable %s\n", name);
+	free(value);
+	out->eprintf(out, "Out of memory setting variable %s\n", name);
 	return err;
     }
     return 0;
@@ -1240,7 +1301,7 @@ ipmi_emu_cmd(emu_out_t *out, emu_data_t *emu, char *cmd_str)
 		    return rv;
 		rv = ipmi_emu_get_mc_by_addr(emu, ipmb, &mc);
 		if (rv) {
-		    out->printf(out, "**Invalid MC address\n");
+		    out->eprintf(out, "**Invalid MC address\n");
 		    return rv;
 		}
 	    }
@@ -1251,7 +1312,7 @@ ipmi_emu_cmd(emu_out_t *out, emu_data_t *emu, char *cmd_str)
 	}
     }
 
-    out->printf(out, "**Unknown command: %s\n", cmd);
+    out->eprintf(out, "**Unknown command: %s\n", cmd);
 
  out:
     return rv;
