@@ -249,13 +249,11 @@ lan_send(lanserv_data_t *lan,
     if (!l)
 	return;
 
+    memset(&msg, 0, sizeof(msg));
     msg.msg_name = &(l->addr);
     msg.msg_namelen = l->addr_len;
     msg.msg_iov = data;
     msg.msg_iovlen = vecs;
-    msg.msg_control = NULL;
-    msg.msg_controllen = 0;
-    msg.msg_flags = 0;
 
     rv = sendmsg(l->xmit_fd, &msg, 0);
     if (rv) {
@@ -370,7 +368,7 @@ lan_channel_init(void *info, channel_t *chan)
     }
 
     if (lan->lan_addr_set) {
-	lan_fd = open_lan_fd(&lan->lan_addr.addr.s_ipsock.s_addr,
+	lan_fd = open_lan_fd(&lan->lan_addr.addr.s_ipsock.s_addr0,
 			     lan->lan_addr.addr_len);
 	if (lan_fd == -1) {
 	    fprintf(stderr, "Unable to open LAN address\n");
@@ -481,7 +479,7 @@ ser_channel_init(void *info, channel_t *chan)
     serserv_data_t *ser = chan->chan_info;
     int err;
     int fd;
-    struct sockaddr *addr = &ser->addr.addr.s_ipsock.s_addr;
+    struct sockaddr *addr = &ser->addr.addr.s_ipsock.s_addr0;
     os_hnd_fd_id_t *fd_id;
     int val;
 
@@ -609,8 +607,8 @@ isim_log(sys_data_t *sys, int logtype, msg_t *msg, const char *format,
 
     con = data->consoles;
     while (con) {
-	con->out.printf(&con->out, "%s", str);
-	con->out.printf(&con->out, "\n");
+	con->out.eprintf(&con->out, "%s", str);
+	con->out.eprintf(&con->out, "\n");
 	con = con->next;
     }
 #if HAVE_SYSLOG
@@ -958,7 +956,7 @@ console_bind_ready(int fd, void *cb_data, os_hnd_fd_id_t *id)
     newcon->shutdown_on_close = 0;
     newcon->telnet = 1;
     newcon->tn_pos = 0;
-    newcon->out.printf = emu_printf;
+    newcon->out.eprintf = emu_printf;
     newcon->out.data = newcon;
 
     setsockopt(rv, IPPROTO_TCP, TCP_NODELAY, (char *)&val, sizeof(val));
@@ -972,6 +970,7 @@ console_bind_ready(int fd, void *cb_data, os_hnd_fd_id_t *id)
 	err = write(rv, msg, strlen(msg));
 	close(rv);
 	free(newcon);
+	return;
     }
 
     newcon->next = misc->consoles;
@@ -997,6 +996,8 @@ init_term(void)
     new_termios.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP
 			     |INLCR|IGNCR|ICRNL|IXON);
     new_termios.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
+    new_termios.c_cc[VTIME] = 0;
+    new_termios.c_cc[VMIN] = 0;
     tcsetattr(0, TCSADRAIN, &new_termios);
 }
 
@@ -1462,10 +1463,10 @@ main(int argc, const char *argv[])
     stdio_console.telnet = 0;
     stdio_console.tn_pos = 0;
     if (nostdio) {
-	stdio_console.out.printf = dummy_printf;
+	stdio_console.out.eprintf = dummy_printf;
 	stdio_console.out.data = &stdio_console;
     } else {
-	stdio_console.out.printf = emu_printf;
+	stdio_console.out.eprintf = emu_printf;
 	stdio_console.out.data = &stdio_console;
     }
     stdio_console.next = NULL;
@@ -1560,7 +1561,7 @@ main(int argc, const char *argv[])
 	int nfd;
 	int val;
 
-	nfd = socket(sysinfo.console_addr.s_ipsock.s_addr.sa_family,
+	nfd = socket(sysinfo.console_addr.s_ipsock.s_addr0.sa_family,
 		     SOCK_STREAM, IPPROTO_TCP);
 	if (nfd == -1) {
 	    perror("Console socket open");
