@@ -510,7 +510,7 @@ lan_return_rsp(channel_t *chan, msg_t *msg, rsp_msg_t *rsp)
 
     return_rsp(lan, msg, NULL, rsp);
 
-    msg = ipmi_mc_get_next_recv_q(chan->mc);
+    msg = ipmi_mc_get_next_recv_q(chan);
     if (!msg)
 	return;
     while (msg) {
@@ -529,7 +529,7 @@ lan_return_rsp(channel_t *chan, msg_t *msg, rsp_msg_t *rsp)
 
 	chan->free(chan, msg);
 
-	msg = ipmi_mc_get_next_recv_q(chan->mc);
+	msg = ipmi_mc_get_next_recv_q(chan);
     }
     if (chan->recv_in_q)
 	chan->recv_in_q(chan, 0);
@@ -643,7 +643,6 @@ handle_get_session_challenge(lanserv_data_t *lan, msg_t *msg)
     uint8_t  data[21];
     user_t   *user;
     uint32_t sid;
-    uint8_t  authtype;
     int      rv;
 
     if (msg->len < 17) {
@@ -653,7 +652,6 @@ handle_get_session_challenge(lanserv_data_t *lan, msg_t *msg)
 	return;
     }
 
-    authtype = msg->data[0] & 0xf;
     user = find_user(lan, msg->data+1, 1, 0);
     if (!user) {
 	lan->sysinfo->log(lan->sysinfo, SESSION_CHALLENGE_FAILED, msg,
@@ -665,13 +663,6 @@ handle_get_session_challenge(lanserv_data_t *lan, msg_t *msg)
 	return;
     }
 
-    if (!(user->allowed_auths & (1 << authtype))) {
-	lan->sysinfo->log(lan->sysinfo, SESSION_CHALLENGE_FAILED, msg,
-		 "Session challenge failed: Invalid authorization type");
-	return_err(lan, msg, NULL, IPMI_INVALID_DATA_FIELD_CC);
-	return;
-    }
-
     if (lan->channel.active_sessions >= MAX_SESSIONS) {
 	lan->sysinfo->log(lan->sysinfo, SESSION_CHALLENGE_FAILED, msg,
 		 "Session challenge failed: To many open sessions");
@@ -895,20 +886,6 @@ handle_temp_session(lanserv_data_t *lan, msg_t *msg)
 		 "Activate session failed: Invalid user idx: 0x%x", user_idx);
 	return;
     }
-    if (! (user->allowed_auths & (1 << auth))) {
-	lan->sysinfo->log(lan->sysinfo, NEW_SESSION_FAILED, msg,
-		 "Activate session failed: Requested auth %d was invalid for"
-		 " user 0x%x",
-		 auth, user_idx);
-	return;
-    }
-    if (! (user->allowed_auths & (1 << msg->authtype))) {
-	lan->sysinfo->log(lan->sysinfo, NEW_SESSION_FAILED, msg,
-		 "Activate session failed: Message auth %d was invalid for"
-		 " user 0x%x",
-		 msg->authtype, user_idx);
-	return;
-    }
 
     if (lan->channel.active_sessions >= MAX_SESSIONS) {
 	lan->sysinfo->log(lan->sysinfo, NEW_SESSION_FAILED, msg,
@@ -2217,7 +2194,7 @@ aes_cbc_encrypt(lanserv_data_t *lan, session_t *session,
     unsigned char  *d;
     unsigned char  *iv;
     unsigned int   i;
-    EVP_CIPHER_CTX ctx;
+    EVP_CIPHER_CTX *ctx;
     int            rv;
     int            outlen;
     int            tmplen;
@@ -2264,14 +2241,18 @@ aes_cbc_encrypt(lanserv_data_t *lan, session_t *session,
     *data_size += 16;
 
     /* Ok, we're set to do the crypt operation. */
-    EVP_CIPHER_CTX_init(&ctx);
-    EVP_EncryptInit_ex(&ctx, EVP_aes_128_cbc(), NULL, a->ckey, iv);
-    EVP_CIPHER_CTX_set_padding(&ctx, 0);
-    if (!EVP_EncryptUpdate(&ctx, *pos, &outlen, d, l)) {
+    ctx = EVP_CIPHER_CTX_new();
+    if (!ctx) {
+	    rv = ENOMEM;
+	    goto out_cleanup;
+    }
+    EVP_EncryptInit_ex(ctx, EVP_aes_128_cbc(), NULL, a->ckey, iv);
+    EVP_CIPHER_CTX_set_padding(ctx, 0);
+    if (!EVP_EncryptUpdate(ctx, *pos, &outlen, d, l)) {
 	rv = ENOMEM;
 	goto out_cleanup;
     }
-    if (!EVP_EncryptFinal_ex(&ctx, (*pos) + outlen, &tmplen)) {
+    if (!EVP_EncryptFinal_ex(ctx, (*pos) + outlen, &tmplen)) {
 	rv = ENOMEM; /* right? */
 	goto out_cleanup;
     }
@@ -2281,7 +2262,7 @@ aes_cbc_encrypt(lanserv_data_t *lan, session_t *session,
     *data_len = outlen + 16;
 
  out_cleanup:
-    EVP_CIPHER_CTX_cleanup(&ctx);
+    EVP_CIPHER_CTX_free(ctx);
     free(d);
     return rv;
 }
@@ -2292,7 +2273,7 @@ aes_cbc_decrypt(lanserv_data_t *lan, session_t *session, msg_t *msg)
     auth_data_t    *a = &session->auth_data;
     unsigned int   l = msg->len;
     unsigned char  *d;
-    EVP_CIPHER_CTX ctx;
+    EVP_CIPHER_CTX *ctx;
     int            outlen;
     unsigned char  *pad;
     int            padlen;
@@ -2312,10 +2293,14 @@ aes_cbc_decrypt(lanserv_data_t *lan, session_t *session, msg_t *msg)
     memcpy(d, msg->data+16, l);
 
     /* Ok, we're set to do the decrypt operation. */
-    EVP_CIPHER_CTX_init(&ctx);
-    EVP_DecryptInit_ex(&ctx, EVP_aes_128_cbc(), NULL, a->k2, msg->data);
-    EVP_CIPHER_CTX_set_padding(&ctx, 0);
-    if (!EVP_DecryptUpdate(&ctx, msg->data+16, &outlen, d, l)) {
+    ctx = EVP_CIPHER_CTX_new();
+    if (!ctx) {
+	    rv = ENOMEM;
+	    goto out_cleanup;
+    }
+    EVP_DecryptInit_ex(ctx, EVP_aes_128_cbc(), NULL, a->k2, msg->data);
+    EVP_CIPHER_CTX_set_padding(ctx, 0);
+    if (!EVP_DecryptUpdate(ctx, msg->data+16, &outlen, d, l)) {
 	rv = EINVAL;
 	goto out_cleanup;
     }
@@ -2348,7 +2333,7 @@ aes_cbc_decrypt(lanserv_data_t *lan, session_t *session, msg_t *msg)
     msg->len = outlen;
 
  out_cleanup:
-    EVP_CIPHER_CTX_cleanup(&ctx);
+    EVP_CIPHER_CTX_free(ctx);
     free(d);
     return rv;
 }
@@ -3191,6 +3176,8 @@ ipmi_lan_init(lanserv_data_t *lan)
     lan->channel.set_associated_mc = set_associated_mc;
     lan->channel.get_associated_mc = get_associated_mc;
 
+    lan->channel.has_recv_q = 1;
+
     /* Force user 1 to be a null user. */
     memset(lan->users[1].username, 0, 16);
 
