@@ -64,7 +64,7 @@ static int   interactive        = 1;
 static int   interactive_done   = 0;
 
 char *progname;
-selector_t *sel;
+struct selector_s *sel;
 os_handler_t *os_hnd;
 static ipmi_con_t *con;
 static int continue_operation = 1;
@@ -676,7 +676,11 @@ main(int argc, char *argv[])
     }
 
     /* Create selector with os handler. */
-    sel_alloc_selector(os_hnd, &sel);
+    rv = sel_alloc_selector_nothread(&sel);
+    if (rv) {
+	fprintf(stderr, "Error allocating selector: 0x%x\n", rv);
+	exit(1);
+    }
 
     /* The OS handler has to know about the selector. */
     ipmi_posix_os_handler_set_sel(os_hnd, sel);
