@@ -40,7 +40,8 @@ class SensorRefreshData:
         return
 
     def sensor_cb(self, sensor):
-        sensor.get_value(self.s)
+        if (sensor.is_readable()):
+            sensor.get_value(self.s)
         return
 
     pass
@@ -307,7 +308,7 @@ class SensorEventEnablesSet:
             if (self.s.is_threshold):
                 labels.append(threshold_event_str_to_full(i))
             else:
-                name = s.events_supported_name[i]
+                name = self.s.events_supported_name[i]
                 labels.append(discrete_event_str_to_full(i, name))
                 pass
             pass
@@ -329,6 +330,8 @@ class SensorEventEnablesSet:
 
 class Sensor:
     def __init__(self, e, sensor):
+        if (e.ui.in_destroy):
+            return
         self.e = e
         self.name = sensor.get_name()
         e.sensors[self.name] = self
@@ -364,7 +367,12 @@ class Sensor:
             pass
 
         sensor.add_event_handler(self)
-        sensor.get_value(self)
+        if (sensor.is_readable()):
+            sensor.get_value(self)
+            pass
+        else:
+            self.ui.set_item_text(self.treeroot, "(not readable)")
+            pass
 
         self.auto_rearm = sensor.get_supports_auto_rearm()
 
@@ -539,7 +547,12 @@ class Sensor:
         return
 
     def HandleMenu(self, event):
-        l = [ ]
+        if (self.impt_data == None):
+            l = [ [ "Add to watch values", self.add_impt ] ]
+            pass
+        else:
+            l = [ [ "Remove from watch values", self.remove_impt ] ]
+            pass
         if (self.event_support != OpenIPMI.EVENT_SUPPORT_NONE):
             l.append( ("Rearm", self.Rearm) )
             pass
@@ -559,6 +572,14 @@ class Sensor:
             pass
         return
 
+    def add_impt(self, event):
+        self.ui.add_impt_data("sensor", str(self), self)
+        return
+        
+    def remove_impt(self, event):
+        self.ui.remove_impt_data(self.impt_data)
+        return
+        
     def Rearm(self, event):
         return
     
